import os
from textwrap import dedent
from flask import Flask, request, make_response, redirect

import base64
from Crypto.Cipher import AES
from Crypto.Util.Padding import pad, unpad
from urllib.parse import parse_qs

KEY = os.urandom(16)
FLAG = open('flag.txt').read()

app = Flask(__name__)

users = {
    'admin': {
        'is_admin': True,
        'password': os.urandom(16).hex() # It is so secure that you cannot break it
    },
    'guest': {
        'is_admin': False,
        'password': 'guest'
    }
}

@app.route("/")
def main():
    token = request.cookies.get('token')
    if not token:
        # Not logged in
        return dedent('''
        <form method='POST'>
        Username: <input name='username'><br>
        Password: <input type='password' name='password'><br>
        <input type='submit' value='Log in!'>
        </form>
        ''')

    try:
        # Logged in
        token = base64.b64decode(token)
        iv, ciphertext = token[:16], token[16:]
        cipher = AES.new(KEY, AES.MODE_CBC, iv)
        token = cipher.decrypt(ciphertext)
        token = unpad(token, 16)
        user = parse_qs(token)

        username = user.get(b'username')[0].decode()
        is_admin = int(user.get(b'is_admin')[0].decode()) == 1

        if is_admin:
            return f'Hello {username}! Since you are an admin, please grab the flag and keep it safe! {FLAG}.'
        else:
            res = make_response(f'Hello {username}!')
            res.set_cookie('token', '', max_age=0)
            return res
    except Exception as e:
        res = make_response(f'Invalid token. Please refresh the page.<br><xmp>{e}</xmp>', 400)
        res.set_cookie('token', '', max_age=0)
        return res
   

@app.route("/", methods=["POST"])
def login():
    username = request.form['username']
    password = request.form['password']

    if users.get(username) is None:
        return 'User not found', 404
    user = users[username]
    if user.get('password') != password:
        return 'Incorrect password', 401

    is_admin = int(user.get('is_admin', 0))

    iv = os.urandom(16)
    payload = f'is_admin={is_admin}&username={username}'.encode()
    cipher = AES.new(KEY, AES.MODE_CBC, iv)

    padded_payload = pad(payload, 16)

    ciphertext = cipher.encrypt(padded_payload)
    token = base64.b64encode(iv + ciphertext).decode()

    res = make_response(redirect("/", code=302))
    res.set_cookie('token', token)
    return res

app.run(host='0.0.0.0', port=8080)